<?php
/*
Plugin Name: WordPress Tweaks
Plugin URI: http://wordpress.jdwebdev.com/plugins/tweaks/
Description: Adds a variety of useful options and settings.
Version: 0.1.2
Author: John Lamansky
Author URI: http://wordpress.jdwebdev.com
*/


/************
* CONSTANTS *
************/

//Set this to 0 to disable the tweak counter in the footer

define("JL_WPT_FOOTER_COUNT_ENABLED", 1);


//Customize the footer text

define("JL_WPT_FOOTER_COUNT_TEXT", "<p id='tweaks'>This blog has been fine-tuned with %s.</p>");


//Set this to 0 to disable CSS styling for the tweak counter in the footer

define("JL_WPT_FOOTER_COUNT_CSS", 1);



/*******************
* JavaScript & CSS *
*******************/

if (isset($_GET['js'])) {
	jl_wpt_javascript();
	die();
} elseif(isset($_GET['css'])) {
	jl_wpt_css();
	die();
} else {
	$jl_wpt = new JL_WordPress_Tweaks();
}


/*************
* MAIN CLASS *
*************/

class JL_WordPress_Tweaks {

	/*******************
	* GLOBAL VARIABLES *
	*******************/

	var $tweaks = array();

	var $submitted_field_name = "jl_wpt_submitted";

	var $options = array(
		'Admin' =>		array(	'admin_menu_rename_blogroll' => 'Rename &#8220;Blogroll&#8221; admin menu item to &#8220;Links&#8221;'),
		'Comments' =>		array(	'comment_author_dofollow' => 'Remove &#8220;nofollow&#8221; from comment author links',
						'comment_body_dofollow' => 'Remove &#8220;nofollow&#8221; from comment body links',
						'comment_targetblank' => 'Open external comment links in new windows'),
		'Fixes' =>		array(	'fix_blogpage_cpi' => 'When using a static page to show blog posts, use &#8220;current_page_item&#8221; CSS class when that page is showing <small>[<a href="http://trac.wordpress.org/ticket/2959" target="_blank">More info</a>]</small>'),
		'Posts and Pages' =>	array(	'post_targetblank' => 'Open external post links in new windows',
						'post_excerpt_readmore' => 'When displaying post excerpts on archive pages, display a &#8220;Read more &raquo;&#8221; link instead of &#8220;[...]&#8221;'),
		'SEO' =>		array(	'archive_excerpts' => 'Show post excerpts (instead of full content) on archive pages',
						'post_morelink_nofollow' => 'Add &#8220;nofollow&#8221; to &#8220;Read more&#8221; links <small>[<a href="http://rillz.com/magazine/wordpress-seo-bug/" target="_blank">More info</a>]</small>',
						'meta_nofollow' => 'Add &#8220;nofollow&#8221; to the &#8220;Register&#8221; and &#8220;Login&#8221; links',
						'widget_tags_homeonly' => 'Only show the tag cloud widget on the homepage (to decrease links per page count)'),
		'Theme and Appearance'=>array(	'page_list_nospace' => 'Remove white space from pages list')
	);

	var $plugin_path;

	/*******
	* INIT *
	*******/

	function __construct() {
		$this->JL_WordPress_Tweaks();
	}

	function JL_WordPress_Tweaks() {

		$this->load_plugin_data();
		$this->plugin_path = get_bloginfo('wpurl').'/'.PLUGINDIR.'/tweaks.php';

		register_activation_hook(__FILE__, array($this, 'activate'));

		if ($this->options_page_submitted()) {

			//Update options
			$option_values = array();
			foreach ($this->options as $options_in_cat) {
				foreach ($options_in_cat as $option_key => $option_text) {
					$option_values[$option_key] = ($_POST[$option_key] == 'on');
				}
			}
			update_option('jl_wpt_tweaks', serialize($option_values));
			$this->tweaks = $option_values;
		} else {
			$this->tweaks = $this->get_tweaks_array();
		}

		add_action('admin_menu', array($this, 'add_the_options_page'));
		add_action('admin_head', array($this, 'options_page_css'));
		add_action('wp_head', array($this, 'include_global_javascript'));

		$filters =array('admin_link_text' => 'register',
				'archive_excerpts' => 'the_content',
				'comment_author_dofollow' => 'get_comment_author_link',
				'comment_body_dofollow' => 'get_comment_text',
				'fix_blogpage_cpi' => 'wp_list_pages',
				'meta_nofollow' => array('register', 'loginout'),
				'post_excerpt_readmore' => 'the_excerpt',
				'post_morelink_nofollow' => 'the_content',
				'page_list_nospace' => 'wp_list_pages',
				'theme_title_tagline' => 'wp_title',
				'widget_tags_homeonly' => 'option_sidebars_widgets');
		$actions =array('admin_menu_rename_blogroll' => '_admin_menu',
				'wpt_footer_count' => 'wp_footer',
				'post_targetblank' => 'wp_footer',
				'comment_targetblank' => 'wp_footer');
		$args = array(	'theme_title_tagline' => 2);

		foreach ($filters as $tweak => $filterArray) {
			if (!is_array($filterArray)) $filterArray = array($filterArray);
			foreach ($filterArray as $filter) {
				if ($this->is_tweak_enabled($tweak)) {
					if (!($arg = $args[$tweak])) $arg = 1;
					add_filter($filter, array($this, $tweak), 10, $arg);
				}
			}
		}

		foreach ($actions as $tweak => $actionArray) {
			if (!is_array($actionArray)) $actionArray = array($actionArray);
			foreach ($actionArray as $action) {
				if ($this->is_tweak_enabled($tweak)) {
					if (!($arg = $args[$tweak])) $arg = 1;
					add_action($action, array($this, $tweak), 10, $arg);
				}
			}
		}

		if (JL_WPT_FOOTER_COUNT_ENABLED == 1) {
			add_action('wp_footer', array(&$this, 'footer_count'));
		}

		if (JL_WPT_FOOTER_COUNT_ENABLED == 1 && JL_WPT_FOOTER_COUNT_CSS == 1) {
			add_action('wp_head', array(&$this, 'include_footer_count_css'));
		}
	}

	function activate() {

		//Nothing here yet
	}


	/*********************
	* SETTINGS FUNCTIONS *
	*********************/

	var $plugin_data = array();

	function load_plugin_data() {

		$keys = array('Plugin Name', 'Plugin URI', 'Description', 'Author', 'Author URI', 'Version');
		$data = array();

		$plugin_data = implode('', file(__FILE__));

		foreach ($keys as $key) {
			preg_match("|$key: (.*)|i", $plugin_data, $value);
			$data[strtolower($key)] = $value[1];
		}

		$this->plugin_data = $data;
	}

	function get_plugin_data($key) {
		return $this->plugin_data[strtolower($key)];
	}

	function get_tweaks_array() {
		if ($tweaks = get_option('jl_wpt_tweaks'))
			return unserialize($tweaks);
		else
			return array();
	}

	function is_tweak_enabled($key) {
		return $this->tweaks[$key];
	}

	function get_tweak_count() {
		return count(array_filter($this->tweaks));
	}

	/*********************
	* ADMIN OPTIONS PAGE *
	*********************/

	function add_the_options_page() {
		add_options_page('WordPress Tweaks', 'Tweaks', 8, __FILE__, array($this, 'show_options_page'));
	}

	function options_page_css() {
		$this->include_css('admin');
	}

	function show_options_page() {

		if ($this->options_page_submitted()) {

			//Update options
			$option_values = array();
			foreach ($this->options as $options_in_cat) {
				foreach ($options_in_cat as $option_key => $option_text) {
					$option_values[$option_key] = ($_POST[$option_key] == 'on');
				}
			}
			update_option('jl_wpt_tweaks', serialize($option_values));
			$this->tweaks = $option_values;

			echo '<div id="message" class="updated fade"><p><strong>' . __('Options saved.', 'jl_wpt') . '</strong></p></div>';
		}
	?>
	<div id="jl_wpt_settings" class="wrap">
	<h2><?php _e($this->get_plugin_data('plugin name'), 'jl_wpt'); echo ' '.$this->get_plugin_data('version'); ?></h2>
	<div class="donate">
		<p>Do you find this plugin useful? Please consider making a donation. Thanks!</p>
		<p class="donate-button"><a href="<?php echo $this->get_plugin_data('plugin uri'); ?>donate" target="_blank"><img src="<?php echo $this->get_plugin_data('author uri'); ?>/donate/images/btn_donate_SM.gif" alt="Donate" /></a></p>
	</div>
	<form name="jl_wpt_options" method="post" action="">
		<input type="hidden" name="<?php echo $this->submitted_field_name; ?>" value="Y" />
	<?php
		foreach ($this->options as $option_cat => $options_in_cat) {
			echo "\t<h3>".__($option_cat, 'jl_wpt')."</h3>\n\t<table>";

			foreach ($options_in_cat as $option_key => $option_text) {
				echo "\t\t<tr valign='top'>\n";
				echo "\t\t\t<td><input type='checkbox' name='$option_key' id='$option_key' ";
				if ($this->is_tweak_enabled($option_key)) echo "checked='checked' ";
				echo "/></td>\n";
				echo "\t\t\t<td>".__($option_text, 'jl_wpt')."</td>";
				echo "\t\t</tr>\n";
			}

			echo "\t</table>\n";
		}
	?>
		<p><span class="submit">
			<input type="submit" name="submit" value="<?php _e('Update Options', 'jl_wpt') ?>" />
		</span></p>
	</form>
	</div>
	<?php
	}

	function options_page_submitted() {
		return ($_POST[$this->submitted_field_name] == "Y");
	}


	/***************
	* FOOTER COUNT *
	***************/

	function footer_count() {
		$count = $this->get_tweak_count();
		if ($count > 0) {
			$pluginUri = $this->get_plugin_data('Plugin URI');
			if ($count > 1) $s = 's'; else $s = '';

			printf(JL_WPT_FOOTER_COUNT_TEXT, "$count <a href=\"$pluginUri\">WordPress Tweak$s</a>");
		}
	}

	function include_footer_count_css() {
		$this->include_css('footer_count');
	}


	/***************
	* HEADER LINKS *
	***************/

	function include_global_javascript() {
		$this->include_javascript('functions');
	}

	//Put JavaScript and CSS links
	function include_javascript($name) {
		echo '<script type="text/javascript" src="'.$this->plugin_path.'?js='.$name.'"></script>'."\n";
	}

	function include_css($name) {
		echo '<link rel="stylesheet" type="text/css" media="screen" href="'.$this->plugin_path.'?css='.$name.'" />'."\n";
	}


	/**************************
	* GENERAL TWEAK FUNCTIONS *
	**************************/

	//Functions used by more than one tweak

	function remove_nofollow($html) {
		return eregi_replace(	'<a ([^>]*)rel=(["\'])([a-zA-Z0-9_ -]*)nofollow([a-zA-Z0-9_ -]*)(["\'])([^>]*)>',
					'<a \\1rel=\\2\\3\\4\\5\\6>', $html);
	}


	/******************
	* TWEAK FUNCTIONS *
	******************/

	function admin_menu_rename_blogroll() {
		global $menu, $submenu;
		$menu[20] = array(__('Links'), 'manage_links', 'link-manager.php');
		$submenu['link-manager.php'][5] = array(__('Manage Links'), 'manage_links', 'link-manager.php');
	}


	function archive_excerpts($content) {
		if (is_archive()) {
			$content = strip_tags($content);
			$maxWords = 55;

			$words = explode(" ", $content);
			if (count($words) > $maxWords) {
				$excerpt = implode(" ", array_slice($words, 0, $maxWords)) . " [...]";
				if ($this->is_tweak_enabled('post_excerpt_readmore'))
					return $this->post_excerpt_readmore($excerpt);
				else
					return $excerpt;
			}
		}

		return $content;
	}

	function comment_author_dofollow($link) {
		return $this->remove_nofollow($link);
	}

	function comment_body_dofollow($comments) {
		return $this->remove_nofollow($comments);

	}

	function comment_targetblank() {
		if (is_single()) {
			$this->include_javascript('comment_targetblank');
		}
	}

	function fix_blogpage_cpi($pagesList) {
		//If we're using a static page for blog posts, and if we're there...
		if (get_option('page_for_posts') && is_home()) {

			//Get the page object
			$pageForPosts = get_page(get_option('page_for_posts'));

			//Escape the dots in the URL so they don't mess up the regex
			$safeGuid = str_replace(array('.'), array('\\.'), $pageForPosts->guid);

			//Add the CSS class and return
			return eregi_replace(	'<li([^>]*) class="([a-zA-Z0-9_ -]+)"([^>]*)>([\s]*)<a href="'.$safeGuid.'"',
						'<li\\1 class="\\2 current_page_item"\\3>\\4<a href="'.$safeGuid.'"',
						$pagesList);
		} else {
			return $pagesList;
		}
	}

	function post_excerpt_readmore($excerpt) {
		if (is_archive()) {
			global $post;
			$permalink = $post->guid;

			$excerpt = rtrim($excerpt);

			if ($this->is_tweak_enabled('post_morelink_nofollow')) $nofollow=' rel="nofollow"'; else $nofollow='';
			$morelink = '<a href="'.$permalink.'" class="more-link"'.$nofollow.'>Read more &raquo;</a>';

			$excerpt = ereg_replace("</p>$", " $morelink</p>", $excerpt);
			$excerpt = ereg_replace("\[&#8230;\] $morelink</p>$", "&#8230; $morelink</p>", $excerpt);
		}

		return $excerpt;
	}

	function meta_nofollow($html) {
		if (strpos($html, 'nofollow') === false)
			return str_replace('<a href=', '<a rel="nofollow" href=', $html);
		else
			return $html;
	}

	function post_targetblank() {
		$this->include_javascript('post_targetblank');
	}

	function post_morelink_nofollow($theContent) {
		if (eregi('<a ([^>]*)rel=["\']([a-zA-Z0-9_ -]*)nofollow([a-zA-Z0-9_ -]*)["\']([^>]*) class=["\']more-link["\']([^>]*)>', $theContent)) {

			//It looks like some other plugin has already added nofollow, so we'll return the post as-is

			return $theContent;

		} else {
			if (eregi('<a ([^>]*)rel=["\']([a-zA-Z0-9_ -]*)["\']([^>]*) class=["\']more-link["\']([^>]*)>', $theContent)) {

				//There's already a rel attribute, but not a nofollow, so we'll add it
				return eregi_replace('rel=["\']', 'rel="nofollow ', $theContent);
			} else {

				//No rel attribute, so we'll add one
				return eregi_replace('class=["\']more-link["\']', 'rel="nofollow" class="more-link"', $theContent);
			}
		}
	}
	
	function page_list_nospace($pagesList) {
		return str_replace(array("\r", "\n", "\t"), '', $pagesList);
	}

	function widget_tags_homeonly($value) {
		if (!is_home() && !is_admin()) {
			$i=1;
			while (isset($value["sidebar-$i"])) {
				$value["sidebar-$i"] = array_values(
					array_filter($value["sidebar-$i"], array($this, "widget_tags_homeonly_filter"))
				);
				$i++;
			}
		}
		return $value;
	}

	function widget_tags_homeonly_filter($value) {
		return (strcmp($value, 'tag_cloud') !== 0);
	}
}






/*************
* JAVASCRIPT *
*************/

function jl_wpt_javascript() {
	
	header('Content-type: text/javascript');

	if ($_GET['js'] == 'post_targetblank') {
?>
function setPostLinkTargets(target) {

	var i, elements;

	var classNames = new Array();
	classNames[0] = "entry-content";
	classNames[1] = "post-content";
	classNames[2] = "entry";

	i=0;
	while (elements == null) {
		elements = getElementsByClassName(classNames[i], "div", document.getElementById("content"));
		i++;
	}

	setLinkTargets(target, elements);
}

setPostLinkTargets("_blank");

<?php } elseif ($_GET['js'] == 'comment_targetblank') { ?>

function setCommentLinkTargets(target) {
	setLinkTargets(target, new Array(document.getElementById("comments")));
}

setCommentLinkTargets("_blank");

<?php } elseif ($_GET['js'] == 'functions') { ?>

function setLinkTargets(target, elements) {
	var i, j, anchors;

	for (i=0; i<elements.length; i++) {

		anchors = elements[i].getElementsByTagName("a");

		for (j=0; j<anchors.length; j++) {
			if (getDomainName(anchors[j].href) != getDomainName(document.location.href))
				anchors[j].target = target;
		}
	}
}


function getDomainName(url) {
	var schemeSuffix = "://";
	return url.substr(0, url.indexOf("/", url.indexOf(schemeSuffix) + schemeSuffix.length));
}

/*
Function Source:
http://www.robertnyman.com/2005/11/07/the-ultimate-getelementsbyclassname/
*/

function getElementsByClassName(className, tag, elm){
	var testClass = new RegExp("(^|\\\\s)" + className + "(\\\\s|$)");
	var tag = tag || "*";
	var elm = elm || document;
	var elements = (tag == "*" && elm.all)? elm.all : elm.getElementsByTagName(tag);
	var returnElements = [];
	var current;
	var length = elements.length;
	for(var i=0; i<length; i++){
		current = elements[i];
		if(testClass.test(current.className)){
			returnElements.push(current);
		}
	}
	return returnElements;
}
<?php
	}
}


/******
* CSS *
******/

function jl_wpt_css() {
	
	header('Content-type: text/css');

	if ($_GET['css'] == 'footer_count') {


// In the CSS below, I use "p#tweaks" instead of "#tweaks" to make it less
// likely that the CSS rules are accidentally overriden by the theme rules

?>

p#tweaks {
	opacity: .6;
	filter: alpha(opacity=60);
	-moz-opacity: .6;

	background-color: #fff;
	border: 0;
	border-top: 1px solid #999;
	color: #000;
	margin: 1em 0 0 0;
	padding: 0.5em 0;
	text-align: center;
	width: 100%;
}

p#tweaks a, p#tweaks a:hover, p#tweaks a:visited {
	color: #000;
	border: 0;
	padding: 0;
	margin: 0;
	display: inline;
	text-decoration: underline;
}


<?php
	} elseif ($_GET['css'] == 'admin') {
?>

#jl_wpt_settings .donate {
	float: right;
	font-size: 0.9em;
	width: 160px;
	text-align: center;
}

#jl_wpt_settings .donate .donate-button a, #jl_wpt_settings .donate .donate-button a img {
	text-decoration: none;
	border: 0;
}

<?php
	}
}
?>